/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         application.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : This is the starting point for the
 *                          application. The game core and other
 *                          essentials are setup here.
 *
 */

#include "application.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                                  MAIN
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Application app; //Create an instance of this application.

int Application::main(int argc, char** argv) {
	
	//Redirect errors to the console.
	CL_ConsoleWindow console("Debug console window", 80, 25);
	console.redirect_stdio();
	
	try {
		cout << "Welcome to Poddie!" << endl;
		
		cout << "Setting up Poddie game core... \t";
		
		//Setup ClanLib core
		CL_SetupCore setup_core;
		
		cout << " Success." << endl;
		cout << "Setting up configuration system... \t";
		
		//Setup config filereader
		ConfigFile config("config/autogen-settings.cfg");
		
		//Read extra input from custom config file
		filebuf extra_config;
		extra_config.open("config/settings.cfg", ios::in);
		istream extra_stream(&extra_config);
		
		//Send input to config and close
		extra_stream>>config;
		extra_config.close();
		
		check_config(config);
		
		cout << " Success." << endl;
		cout << "Setting up display system... \t";
		
		//Setup the graphics system
		CL_SetupDisplay setup_display;
		//CL_SetupGL setup_gl;
		CL_SetupGL setup_sdl;
		
		// Setup the GUI system
		CL_SetupGUI setup_gui;
		setup_gui.init();
		
		cout << " Success." << endl;
		cout << "Setting up sound system... \t";
		
		//Setup the sound system.
		CL_SetupSound setup_sound;
		CL_SetupVorbis setup_vorbis;
		CL_SoundOutput output(44100);
		setup_vorbis.init();
		
		cout << " Success." << endl;
		cout << "Setting up networking... \t";
		
		//Setup the networking
		CL_SetupNetwork setup_network;
		
		cout << " Success." << endl;
		cout << "Initiating resource manager... \t";
		
		//Initialize the resource manager
		ostringstream ss;
		ss << "resources/" << config.read<string>("graphics_quality") << "_quality/resources.xml";
		CL_ResourceManager resources(ss.str());
		
		cout << " Success." << endl;
		cout << "Creating game window... \t";
		
		//Create the game window and set fullscreen to false.
		CL_DisplayWindow window("Poddie", config.read<int>("screen_width"), config.read<int>("screen_height"), config.read<int>("fullscreen"));
		
		cout << " Success." << endl;
		cout << "Creating game world... \t";
		
		//Create the world
		World world(&resources, &window, config, true); //True = debug messages on.
		
		cout << " Success." << endl;
		
		//Connect the Window close event
		CL_Slot slot_quit = window.sig_window_close().connect(&world, &World::on_quit);
		
		cout << "Starting game world..." << endl;
		
		try {
			world.run(); // Begin the game loop.
		}
		catch (Exception &e) {
			cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
		}
		
		cout << "Game stopped." << endl;
	}
	catch (CL_Error err) {
		cout << "Exeception caught: " << err.message.c_str() << endl;
		console.wait_for_key();
	}
	
	return 0;
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Application::check_config(ConfigFile &config) {
	// Graphics Configuration
	if (!config.keyExists("screen_width")) config.add("screen_width", 800);
	if (!config.keyExists("screen_height")) config.add("screen_height", 600);
	if (!config.keyExists("fullscreen")) config.add("fullscreen", 0);
	if (!config.keyExists("graphics_quality")) config.add("graphics_quality", "high");
	if (!config.keyExists("FPS")) config.add("FPS", "30");
	
	// Sound Configuration
	if (!config.keyExists("sfx_volume")) config.add("sfx_volume", "1.0");
	
	// Controls Configuration
	if (!config.keyExists("rusty_engine_backcenter1_toggle")) config.add("rusty_engine_backcenter1_toggle", "2");
	if (!config.keyExists("rusty_engine_backcenter1_increase")) config.add("rusty_engine_backcenter1_increase", "w");
	if (!config.keyExists("rusty_engine_backcenter1_decrease")) config.add("rusty_engine_backcenter1_decrease", "s");
	if (!config.keyExists("rusty_engine_backright1_toggle")) config.add("rusty_engine_backright1_toggle", "3");
	if (!config.keyExists("rusty_engine_backright1_increase")) config.add("rusty_engine_backright1_increase", "e");
	if (!config.keyExists("rusty_engine_backright1_decrease")) config.add("rusty_engine_backright1_decrease", "d");
	if (!config.keyExists("rusty_engine_backleft1_toggle")) config.add("rusty_engine_backleft1_toggle", "1");
	if (!config.keyExists("rusty_engine_backleft1_increase")) config.add("rusty_engine_backleft1_increase", "q");
	if (!config.keyExists("rusty_engine_backleft1_decrease")) config.add("rusty_engine_backleft1_decrease", "a");
	if (!config.keyExists("rusty_engine_middleright1_toggle")) config.add("rusty_engine_middleright1_toggle", "5");
	if (!config.keyExists("rusty_engine_middleright1_increase")) config.add("rusty_engine_middleright1_increase", "t");
	if (!config.keyExists("rusty_engine_middleright1_decrease")) config.add("rusty_engine_middleright1_decrease", "g");
	if (!config.keyExists("rusty_engine_middleleft1_toggle")) config.add("rusty_engine_middleleft1_toggle", "4");
	if (!config.keyExists("rusty_engine_middleleft1_increase")) config.add("rusty_engine_middleleft1_increase", "r");
	if (!config.keyExists("rusty_engine_middleleft1_decrease")) config.add("rusty_engine_middleleft1_decrease", "f");
	if (!config.keyExists("rusty_weapon_laser_middleright1_shoot")) config.add("rusty_weapon_laser_middleright1_shoot", "space");
	if (!config.keyExists("rusty_weapon_laser_middleright1_increase")) config.add("rusty_weapon_laser_middleright1_increase", "y");
	if (!config.keyExists("rusty_weapon_laser_middleright1_decrease")) config.add("rusty_weapon_laser_middleright1_decrease", "h");
	if (!config.keyExists("rusty_weapon_laser_middleleft1_shoot")) config.add("rusty_weapon_laser_middleleft1_shoot", "space");
	if (!config.keyExists("rusty_weapon_laser_middleleft1_increase")) config.add("rusty_weapon_laser_middleleft1_increase", "u");
	if (!config.keyExists("rusty_weapon_laser_middleleft1_decrease")) config.add("rusty_weapon_laser_middleleft1_decrease", "j");
}
