/* Copyright (C) 2009 Niels Serup
 * 
 * This file is part of Poddie.
 * 
 * Poddie is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License can be found in the
 * file LICENSE.
 */

/*
 *                         ComponentsMenu.cpp  -  description
 *                         --------------------------------
 *   begin                : Fri Sep 4 2009
 *   copyright            : (C) 2009 by Niels Serup
 *   maintainer(s)        : Niels Serup (nqpz) <ns@metanohi.org>
 *   description          : Controls in-game components menu.
 *
 */

#include "componentsmenu.h"
#include "precomp.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

//ComponentsMenu constructor
ComponentsMenu::ComponentsMenu(World *world, CL_GUIManager *gui) : world(world), gui(gui) {
	// Prepare slots
	slots = new CL_SlotContainer;
	
	
	// Load components info
	ifstream comp_file("resources/poddie-components.list");
	string line, file_contents;
	int lines;
	
	istringstream iss;
	string main, type, detailed;
	int depth;
	
	CL_Button *temp_button;
	
	file_contents = "";
	
	while (!comp_file.eof()) {
		getline(comp_file, line);
		if (line == "")
			continue;
		
		file_contents += line + "\n";
		lines++;
	}
	
	comp_file.close();
	
	iss.str(file_contents);
	
	
	for (int i = 0; i < lines; i++) {
		main = "";
		type = "";
		detailed = "";
		depth = 0;
		
		iss>>main;
		iss>>type;
		iss>>detailed;
		iss>>depth;
		
		temp_button = new CL_Button(CL_Rect(200, i*25, 400, i*25+20), main+" "+type+" "+detailed, gui);
		buttons.push_back(temp_button);
		buttons[i] = new CL_Button(CL_Rect(200, i*25, 400, i*25+20), main+" "+type+" "+detailed, gui);
		temp_button->close();
		comp_entries.push_back(ComponentEntry(main, type, detailed, depth));
		}
	
	
	// Menu isn't shown now
	active = false;
	increase_visibility = false;
	full_visibility = false;
	visibility = 0.0;
	
	gui->disable_input();
	
	ready = false;
	/// IMPORTANT: update() must be run!
	update();
	prepare();
}

void ComponentsMenu::prepare() {
	for (int i = 0; i < buttons.size(); i++) {
		slots->connect(buttons[i]->sig_clicked(), this, &ComponentsMenu::click_component_button, i);
	}
}

//ComponentsMenu destructor
ComponentsMenu::~ComponentsMenu() {
	delete slots;
}

//----------------------------------------------------------------------
//!                             UPDATE
//----------------------------------------------------------------------

void ComponentsMenu::update() {
	// Get size
	int window_width = world->window->get_width();
	int window_height = world->window->get_height();
	menu_top = window_height / 25;
	if (menu_top < 6) menu_top = 6;
	menu_bottom = window_height - menu_top;
	menu_left = window_width / 6;
	if (menu_left < 6) menu_left = 6;
	menu_right = window_width - menu_left;
	
	// Adjust window rect to size of window
	window_rect.top = 0;
	window_rect.bottom = window_height;
	window_rect.left = 0;
	window_rect.right = window_width;
	
	// Apply new size to menu
	menu_rect.top = menu_top;
	menu_rect.bottom = menu_bottom;
	menu_rect.left = menu_left;
	menu_rect.right = menu_right;
	
	for (int i = 0; i < buttons.size(); i++) {
		buttons[i]->set_position(CL_Rect(menu_left + 5, menu_top + 6 + i * 25, menu_right - 5, menu_top + 26 + i * 25));
	}
}

//----------------------------------------------------------------------
//!                             MAGIC
//----------------------------------------------------------------------

void ComponentsMenu::enter() {
	active = true;
	increase_visibility = true;
	full_visibility = false;
	visibility = 0.0;
	
	ready = true;
	
	gui->enable_input();
}

void ComponentsMenu::leave() {
	full_visibility = false;
	increase_visibility = false;
	
	gui->disable_input();
}

void ComponentsMenu::change_state() {
	if (active)
		leave();
	else
		enter();
}

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                              CLICKING
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void ComponentsMenu::click_component_button(int index) {
	if (!ready) return;
	
	int depth = comp_entries[index].depth;
	if (!world->player.at(world->i_control)->slot[depth])
		world->network_component_add_send(world->player.at(world->i_control)->id, comp_entries[index].main, comp_entries[index].type, comp_entries[index].detailed, depth);
	else
		world->network_component_remove_send(world->player.at(world->i_control)->id, depth);
	
	// Focus parent component to avoid focus of current component
	gui->get_root_parent()->set_focus();
}

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                               DRAWING
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void ComponentsMenu::draw() {
	if (!active) return; // Only draw if active
	
	// Background
	CL_Display::fill_rect(window_rect, CL_Color(0,0,0,100*visibility));
	
	// Odd gradient
	CL_Gradient gradient(
		CL_Color(128, 128, 128, 150*visibility),
		CL_Color(128, 128, 128, 150*visibility),
		CL_Color(255, 255, 255, 150*visibility),
		CL_Color(255, 255, 255, 150*visibility)
	);
	
	// Menu rectangle
	CL_Display::fill_rect(menu_rect, gradient);
	CL_Display::draw_rect(menu_rect, CL_Color(0, 0, 0, 255*visibility));
	
	     // ´´´´´´´´´´´´´´´´´´´´´´´´´´´´ //
	    //    Actually _show_ the GUI   //
	if (full_visibility) gui->show();  //
	  //                              //
	 //////////////////////////////////
	
	
	// Check if leaving, arriving or staying
	if (increase_visibility) {
		if (!full_visibility) {
			visibility += .2;
			if (visibility > 1.0) {
				visibility = 1.0;
				full_visibility = true;
			}
		}
	}
	else {
		visibility -= .2;
		if (visibility < 0.0) {
			visibility = 0.0;
			active = false;
		}
	}
}
