/* Copyright (C) 2009 Niels Serup
 * 
 * This file is part of Poddie.
 * 
 * Poddie is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * A copy of the GNU General Public License can be found in the
 * file LICENSE.
 */

/*
 *                         gamemenu.cpp  -  description
 *                         --------------------------------
 *   begin                : Fri Sep 4 2009
 *   copyright            : (C) 2009 by Niels Serup
 *   maintainer(s)        : Niels Serup (nqpz) <ns@metanohi.org>
 *   description          : Controls in-game game menu.
 *
 */

#define COLOR_SPEED 10
#define NUMBER_OF_TABS 2
#define NUMBER_OF_BARS 10 // For volume adjustments

#include "gamemenu.h"
#include "componentsmenu.h"
#include "precomp.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

//GameMenu constructor
GameMenu::GameMenu(World *world, CL_GUIManager *gui, ComponentsMenu *comp_menu) : world(world), gui(gui), comp_menu(comp_menu) {
	short i, j;
	ostringstream ss;
	
	// New manager
	manager = new CL_ComponentManager("resources/GUI/gamemenu-settings.xml", gui);
	
	// Prepare slots
	slots = new CL_SlotContainer;
	
	///                          ///
	/// Get necessary components ///
	///                          ///
	
	/// General components
	manager->get_component("parent", &parent);
	
	manager->get_component("bottom_components", &bottom_components);
	manager->get_component("restart_label", &restart_label);
	manager->get_component("exit_button", &button_exit);
	
	/// Tabs and subtabs	
	// Define number of subtabs
	subtabs_amount.push_back(4); // Tab 0
	subtabs_amount.push_back(2); // Tab 1
	
	string ss0_str, ss1_str;
	for (i = 0; i < NUMBER_OF_TABS; i++) {
		ss.str("");
		ss<<"tab"<<i;
		ss0_str = ss.str();
		
		tab.push_back((CL_Button *) manager->get_component(ss0_str));
		tab_content.push_back(manager->get_component(ss0_str+"_content"));
		
		subtab.push_back(vector<CL_Button*>(subtabs_amount[i]));
		subtab_content.push_back(vector<CL_Component*>(subtabs_amount[i]));
		
		slots->connect(tab[i]->sig_clicked(), this, &GameMenu::click_tab, i);
		
		tab_content[i]->show(false);
		
		for (j = 0; j < subtabs_amount[i]; j++) {
			ss.str("");
			ss<<ss0_str<<"_"<<j;
			ss1_str = ss.str();
			
			subtab[i][j] = (CL_Button *) manager->get_component(ss1_str);
			subtab_content[i][j] = manager->get_component(ss1_str + "_content");
			
			slots->connect(subtab[i][j]->sig_clicked(), this, &GameMenu::click_subtab, j);
			
			subtab_content[i][j]->show(false);
		}
	}
	
	/// General/Controls
	controls_tree = new CL_TreeView(subtab_content[0][1]);
	manager->get_component("change_key", &change_key);
	manager->get_component("new_key", &new_key);
	manager->get_component("save_key", &save_key);
	
	/// General/Video
	manager->get_component("resolution", &resolution_box);
	manager->get_component("new_resolution", &new_res);
	manager->get_component("new_res_width", &new_res_width);
	manager->get_component("new_res_height", &new_res_height);
	manager->get_component("save_res_use", &save_res_use);
	manager->get_component("save_res", &save_res);
	
	manager->get_component("fullscreen", &fullscreen_toggle);
	manager->get_component("windowed", &windowed_toggle);
	
	manager->get_component("quality", &quality_box);
	
	manager->get_component("new_max_fps", &new_max_fps);
	manager->get_component("save_max_fps", &save_max_fps);
	
	/// General/Audio
	manager->get_component("sfx_label", &sfx_label);
	manager->get_component("sfx_volume", &sfx_volume);
	manager->get_component("sfx_adjust", &sfx_adjust);
	
	
	///                ///
	/// Starting it up ///
	///                ///
	
	/// Tabs and subtabs
	// Initiate tabs and subtabs
	current_tab = 0;
	for (i = 0; i < NUMBER_OF_TABS; i++) {
		current_subtab.push_back(0);
	}
	
	click_tab(0);
	click_subtab(0);
	
	/// General/Controls
	// Fill tree up with actions and keys
	controls_tree->set_position(0, 20);
	controls_tree->add_column("Action", 300);
	controls_tree->add_column("Key", 100);
	controls_tree->set_size(400, 400);
	
	slots->connect(controls_tree->sig_selection_changed(), this, &GameMenu::click_controls_item);
	//slots->connect(controls_tree->sig_clicked(), this, &GameMenu::click_controls_item);
		
	char ctrlstr[] = "resources/GUI/controls.text";
	load_controls(ctrlstr);
	
	for (i = 0; i < controls_list.size(); i++) {
		controls_tree->insert_item(
			controls_list[i].action,
			controls_list[i].key
		);
	}
	
	// Don't show "Change key" dialog window
	change_key->show(false);
	
	restart_bacause_of_key_change = false;
		
	/// General/Video
	char resstr[] = "config/available-resolutions.list";
	load_available_resolutions(resstr);
	// Make it possible to add custom resolutions
	resolution_box->insert_item("New resolution");
	
	// Highlight current values
	highlight_resolution_match();	
	highlight_quality_match();
	
	// Don't show "New resolution" dialog window
	new_res->show(false);
	
	// Check a checkbox
	if (world->window->is_fullscreen())
		fullscreen_toggle->set_checked(true);
	else
		windowed_toggle->set_checked(true);
	
	// Write max fps to input box
	new_max_fps->set_text(world->config.read<string>("FPS"));
	
	/// General/Audio
	for (i = 0; i < NUMBER_OF_BARS; i++) {
		sfx_boxes.push_back(CL_Rect());
	}
	
	// Place objects correctly
	sfx_len = (int) sfx_volume->get_width() * world->sfx_volume;
	sfx_bars_used = (int) NUMBER_OF_BARS * world->sfx_volume;
	sfx_adjust->set_position(sfx_len, sfx_adjust->get_client_y());
	update_sfx_label(world->sfx_volume);
	
	/// Various
	// Self-explanatory
	restart_label->show(false);
	
	// Menu isn't shown now
	active = false;
	increase_visibility = false;
	full_visibility = false;
	visibility = 0.0;
	
	// Start values for the fancy color-changing backgrounds
	color_r = 0;
	color_g = 255;
	color_b = 0;
	
	// Not currently adjusting sfx volume
	move_active = false;
	
	new_graphics_quality = world->config.read<string>("graphics_quality");
	
	/// Signals
	slots->connect(button_exit->sig_clicked(), this, &GameMenu::leave);
	
	slots->connect(save_key->sig_clicked(), this, &GameMenu::save_controls_key);
	
	slots->connect(resolution_box->sig_highlighted(), this, &GameMenu::adjust_resolution);
	slots->connect(save_res->sig_clicked(), this, &GameMenu::save_new_resolution, false);
	slots->connect(save_res_use->sig_clicked(), this, &GameMenu::save_new_resolution, true);
	slots->connect(fullscreen_toggle->sig_clicked(), this, &GameMenu::set_fullscreen, true);
	slots->connect(windowed_toggle->sig_clicked(), this, &GameMenu::set_fullscreen, false);
	
	slots->connect(quality_box->sig_highlighted(), this, &GameMenu::adjust_quality);
	
	slots->connect(new_max_fps->sig_got_focus(), this, &GameMenu::click_max_fps_box);
	slots->connect(new_max_fps->sig_lost_focus(), this, &GameMenu::check_click_max_fps_box);
	slots->connect(save_max_fps->sig_clicked(), this, &GameMenu::save_use_max_fps);
	
	slots->connect(sfx_adjust->sig_mouse_down(), this, &GameMenu::start_move);
	slots->connect(sfx_adjust->sig_mouse_up(), this, &GameMenu::end_move);
	
	gui->disable_input();
	
	/// IMPORTANT: update() must be run!
	update();
}

//GameMenu destructor
GameMenu::~GameMenu() {
	save_settings();
	
	// Make sure to throw away segfault-causing variables
	tab.clear();
	tab_content.clear();
	subtab.clear();
	subtab_content.clear();
	
	delete slots;
	delete manager;
}

//----------------------------------------------------------------------
//!                             UPDATE
//----------------------------------------------------------------------

void GameMenu::update() {
	// Get size
	int window_width = world->window->get_width();
	int window_height = world->window->get_height();
	menu_top = window_height / 10;
	if (menu_top < 6) menu_top = 6;
	menu_bottom = window_height - menu_top;
	menu_left = window_width / 10;
	if (menu_left < 6) menu_left = 6;
	menu_right = window_width - menu_left;
	
	// Adjust window rect to size of window
	window_rect.top = 0;
	window_rect.bottom = window_height;
	window_rect.left = 0;
	window_rect.right = window_width;
	
	// Apply new size to menu
	menu_rect.top = menu_top;
	menu_rect.bottom = menu_bottom;
	menu_rect.left = menu_left;
	menu_rect.right = menu_right;
	
	// Calculate the radius of triangles
	if (menu_top < menu_left)
		triangle_radius = menu_top;
	else
		triangle_radius = menu_left;
	triangle_radius -= 5;
	
	// Set position according to menu rectangle
	parent->set_position(menu_rect);
	
	// Look for submenus that have elements that are supposed to be
	// shown on the right side. Adjust if possible.
	short i, j, right_x;
	right_x = (menu_right - menu_left) / 2;
	
	for (i = 0; i < NUMBER_OF_TABS; i++) {
		for (j = 0; j < subtabs_amount[i]; j++) {
			ostringstream ss;
			CL_Component *temp_comp;
			ss<<"tab"<<i<<"_"<<j<<"_right";	
			try {
				temp_comp = manager->get_component(ss.str());
				temp_comp->set_position(right_x, 0);
			}
			catch(CL_Error err) {}
		}
	}
	
	// Make tree take up all menu space
	controls_tree->set_size(menu_right - menu_left - 40, menu_bottom - menu_top - 170);
	
	// Adjust bottom
	bottom_components->set_position(0, menu_bottom - menu_top - 40);
	
	
	
	int sfx_l, sfx_t, sfx_w, sfx_h, bar_width, bar_width_raw;
	sfx_l = sfx_volume->get_screen_x();
	sfx_t = sfx_volume->get_screen_y();
	sfx_w = sfx_volume->get_width();
	sfx_h = sfx_volume->get_height();
	
	bar_width = (sfx_w - NUMBER_OF_BARS) / NUMBER_OF_BARS;
	bar_width_raw = bar_width + 1;
		
	for (i = 0; i < NUMBER_OF_BARS; i++) {
		sfx_boxes[i].left = sfx_l + bar_width_raw*i + sfx_adjust->get_width()/2;
		sfx_boxes[i].top = sfx_t + (NUMBER_OF_BARS-1-i)*5;
		sfx_boxes[i].right = sfx_boxes[i].left + bar_width;
		sfx_boxes[i].bottom = sfx_t + sfx_h;
	}
	
	// Also remember to update the components menu.
	comp_menu->update();
}

//----------------------------------------------------------------------
//!                             MAGIC
//----------------------------------------------------------------------

void GameMenu::enter() {
	active = true;
	increase_visibility = true;
	full_visibility = false;
	visibility = 0.0;
	
	gui->enable_input();
}

void GameMenu::leave() {
	full_visibility = false;
	increase_visibility = false;
	
	// Focus parent component to avoid focus of other components
	parent->set_focus();
	
	gui->disable_input();
}

void GameMenu::change_state() {
	if (active)
		leave();
	else
		enter();
}

//----------------------------------------------------------------------
//!                             TABS
//----------------------------------------------------------------------

//                                TAB
void GameMenu::click_tab(short tab_id) {
	if (current_tab != tab_id) {
		current_tab = tab_id;
		click_subtab(current_subtab[current_tab]);
	}
	
	short i;
	for (i = 0; i < NUMBER_OF_TABS; i++) {
		tab_content[i]->show(false);
		tab[i]->set_down(false);
	}
	
	tab[tab_id]->set_down();
	tab_content[tab_id]->show();
}

//                                SUBTAB
void GameMenu::click_subtab(short subtab_id) {
	current_subtab[current_tab] = subtab_id;
	
	short i, j;
	for (i = 0; i < NUMBER_OF_TABS; i++) {
		for (j = 0; j < subtabs_amount[i]; j++) {
			subtab_content[i][j]->show(false);
			subtab[i][j]->set_down(false);	
		}	
	}
	
	subtab[current_tab][subtab_id]->set_down();
	subtab_content[current_tab][subtab_id]->show();
}

//----------------------------------------------------------------------
//!                            CONTROLS
//----------------------------------------------------------------------

//                                LOAD
void GameMenu::load_controls(char file[]) {
	string line, action, technical;
	short first_char;
	ifstream controls_file(file);
	
	while (!controls_file.eof()) {
		getline(controls_file, line);
		
		first_char = 0;
		while (line[first_char] == ' ')
			first_char++;
		
		
		if (line == "" || line[first_char] == '#')
			continue;
		
		if (action == "")
			action = line;
		else if (technical == "")
			technical = line;
		
		if (action != "" && technical != "") {
			controls_list.push_back(ControlsEntry(
				action, 
				technical,
				world->config.read<string>(technical)
			));
			
			action = "";
			technical = "";
		}
	}
	
	controls_file.close();
}

//                           ITEM CLICKED
void GameMenu::click_controls_item(const CL_TreeNode& node) {
	// Why the hell does it has to be this stupid?
	current_item = reinterpret_cast<CL_TreeItem*>(node.get_component());
	
	new_key->set_text("");
	controls_tree->lower();
	
	new_key->set_focus();
	change_key->show(true);
}

//                             NEW KEY
void GameMenu::save_controls_key() {
	string k, v;
	
	k = new_key->get_text();
	if (k == " ")
		k = "space";
	
	// Get descriptive text
	v = current_item->get_text(0);
	
	for (int i = 0; i < controls_list.size(); i++) {
		if (v == controls_list[i].action) {
			controls_list[i].key = k;
			break;
		}
	}
	
	//world->config.add(v, k);
	current_item->set_text(1, k);
	
	change_key->show(false);
	
	restart_bacause_of_key_change = true;
	restart_label->show(true);
}

//----------------------------------------------------------------------
//!                           RESOLUTION
//----------------------------------------------------------------------

//                                 LOAD
void GameMenu::load_available_resolutions(char file[]) {
	string line;
	short first_char;
	ifstream res_file(file);
	
	while (!res_file.eof()) {
		getline(res_file, line);
		if (line != "") {
			available_resolutions.push_back(line);
			resolution_box->insert_item(line);
		}
	}
	
	res_file.close();
}

//                                ADJUST
void GameMenu::adjust_resolution(int focus) {
	CL_ListItem *temp_list_item;
	string res;
	istringstream iss;
	size_t res_x;
	short res_width, res_height;
	
	new_res->show(false);
	temp_list_item = resolution_box->get_item(focus);
	res = temp_list_item->str;
	res_x = res.find('x');
	if (res_x != string::npos) {
		iss.str(res.substr(0, res_x) + " " + res.substr(res_x+1));
		iss>>res_width;
		iss>>res_height;
		world->window->set_size(res_width, res_height);
		set_fullscreen(world->window->is_fullscreen());
		update();
	}
	else if (res == "New resolution") {
		new_res_width->clear();
		new_res_height->clear();
		new_res->show(true);
	}
}

//                                ADD
void GameMenu::save_new_resolution(bool use) {
	if (new_res_width->get_text() == "" || new_res_height->get_text() == "") return;
	
	string new_res_str = new_res_width->get_text() + "x" + new_res_height->get_text();
	
	available_resolutions.push_back(new_res_str);
	
	resolution_box->remove_item(resolution_box->get_count()-1);
	resolution_box->insert_item(new_res_str);
	resolution_box->insert_item("New resolution");
	
	new_res->show(false);
	
	if (use)
		adjust_resolution(resolution_box->get_count() - 2);
	
	highlight_resolution_match();
}

//                                HIGHLIGHT
void GameMenu::highlight_resolution_match() {
	CL_ListItem *tempitem;
	string cmpstr, tempstr;
	int i;
	bool ok = false; // Not good
	
	// Get resolution string (e.g. 640x480)
	ostringstream ss;
	ss<<world->window->get_width()<<"x"<<world->window->get_height();
	cmpstr = ss.str();
	
	// Loop through resolution items
	for (i = 0; i < resolution_box->get_count(); i++) {
		tempitem = resolution_box->get_item(i);
		tempstr = tempitem->str;
		if (tempstr == cmpstr) {
			resolution_box->set_selected(i, true);
			ok = true;
			break;
		}
	}
	if (!ok) {
		// If the perfect resolution string doesn't exist, create it!
		resolution_box->remove_item(resolution_box->get_count()-1);
		resolution_box->insert_item(cmpstr);
		resolution_box->insert_item("New resolution");
		resolution_box->set_selected(resolution_box->get_count()-2, true);
	}
}

//----------------------------------------------------------------------
//!                        GRAPHICS QUALITY
//----------------------------------------------------------------------

//                                ADJUST
void GameMenu::adjust_quality(int focus) {
	string new_quality;
	if (focus == 0)
		new_quality = "high";
	else if (focus == 1)
		new_quality = "low";
	else if (focus == 2)
		new_quality = "verylow";
	
	if (world->config.read<string>("graphics_quality") != new_quality)
		restart_label->show(true);
	else if (!restart_bacause_of_key_change)
		restart_label->show(false);
	
	new_graphics_quality = new_quality;
}

//                                HIGHLIGHT
void GameMenu::highlight_quality_match() {
	int focus;
	string tempstr(world->config.read<string>("graphics_quality"));
	
	if (tempstr == "high")
		focus = 0;
	else if (tempstr == "low")
		focus = 1;
	else if (tempstr == "verylow")
		focus = 2;
	
	quality_box->set_selected(focus, true);
}

//----------------------------------------------------------------------
//!                        VOLUME ADJUSTMENTS
//----------------------------------------------------------------------

void GameMenu::start_move(const CL_InputEvent& event) {
	move_active = true;
	mouse_start_x = sfx_adjust->get_screen_x() + event.mouse_pos.x - sfx_adjust->get_client_x();
	//cout<<mouse_start_x<<endl;
}

void GameMenu::end_move(const CL_InputEvent& event) {
	move_active = false;
	//cout<<event.mouse_pos.x<<","<<event.mouse_pos.y<<endl;
}

//----------------------------------------------------------------------
//!                            VARIOUS
//----------------------------------------------------------------------

void GameMenu::set_fullscreen(bool yes) {
	if (yes) {
		CL_Display::set_fullscreen(world->window->get_width(), world->window->get_height(), 24);
		windowed_toggle->set_checked(false);
		fullscreen_toggle->set_checked(true);
	}
	else {
		CL_Display::set_windowed();
		windowed_toggle->set_checked(true);
		fullscreen_toggle->set_checked(false);
	}
	
	// Update minimap on new resolution
	world->update_minimap();
}

void GameMenu::update_sfx_label(float volume) {
	int vol100 = (int) (volume * 100.0);
	ostringstream voltxt;
	voltxt<<"SFX volume: "<<vol100<<"/100";
	sfx_label->set_text(voltxt.str());	
}

void GameMenu::click_max_fps_box() {
	temp_max_fps_str = new_max_fps->get_text();
	new_max_fps->set_text("");
}

void GameMenu::check_click_max_fps_box() {
	if (new_max_fps->get_text() == "")
		new_max_fps->set_text(temp_max_fps_str);
	else
		save_use_max_fps();
}

void GameMenu::save_use_max_fps() {
	if (new_max_fps->get_text() == "") {
		new_max_fps->set_text(temp_max_fps_str);
		return;
	}
	
	float fps;
	istringstream iss;
	iss.str(new_max_fps->get_text());
	iss>>fps;
	
	world->FPS = fps;
}

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                           SAVING SETTINGS
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void GameMenu::save_settings() {
	
	// Check for bogus game menu. It's odd.
	int size, i;
	size = controls_list.size();
	if (size == 0)
		return;
	
	ofstream autogen, availres;
	autogen.open("config/autogen-settings.cfg");
	for (i = 0; i < size; i++) {
		autogen<<controls_list[i].technical<<"="<<controls_list[i].key<<"\n";
	}
	
	autogen<<"screen_width="<<CL_Display::get_width()<<"\n";
	autogen<<"screen_height="<<CL_Display::get_height()<<"\n";
	
	if (CL_Display::is_fullscreen())
		autogen<<"fullscreen=1\n";
	
	autogen<<"graphics_quality="<<new_graphics_quality<<"\n";
	autogen<<"FPS="<<world->FPS<<"\n";
	
	autogen<<"sfx_volume="<<world->sfx_volume<<"\n";	
	autogen.close();
	
	size = available_resolutions.size();
	availres.open("config/available-resolutions.list");
	for (i = 0; i < size; i++) {
		availres<<available_resolutions[i]<<"\n";
	}
	availres.close();
}

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                               DRAWING
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void GameMenu::draw() {
	if (!active) return; // Only draw if active
	
	/// Pre-drawing checks START
	if (move_active) {
		int new_x = CL_Mouse::get_x() - mouse_start_x;
		int pw = sfx_volume->get_width();
		if (new_x < 0)
			new_x = 0;
		else if (new_x > pw)
			new_x = pw;
		
		sfx_len = new_x;
		sfx_adjust->set_position(new_x, sfx_adjust->get_client_y());
		
		float volume = (float)sfx_len / (float)pw;
		
		world->sfx_volume = volume;
		
		sfx_bars_used = (int) (volume * (float)NUMBER_OF_BARS);
		
		update_sfx_label(volume);
	}
	/// Pre-drawing checks END
	
	// Background
	CL_Display::fill_rect(window_rect, CL_Color(0,0,0,100*visibility));
	
	// Odd gradient
	CL_Gradient gradient(
		CL_Color(color_r, color_g, color_b, 150*visibility),
		CL_Color(color_g, color_b, color_r, 150*visibility),
		CL_Color(color_b, color_r, color_g, 150*visibility),
		CL_Color(0, 0, 0, 150*visibility)
	);
	
	// Fancy triangles
	double x1,x2,x3, y1,y2,y3, temp_angle;
	
	x1=sin(angle*DEG_TO_RAD)*triangle_radius;
	y1=cos(angle*DEG_TO_RAD)*triangle_radius;
	x2=sin((angle+120)*DEG_TO_RAD)*triangle_radius;
	y2=cos((angle+120)*DEG_TO_RAD)*triangle_radius;
	x3=sin((angle+240)*DEG_TO_RAD)*triangle_radius;
	y3=cos((angle+240)*DEG_TO_RAD)*triangle_radius;
	
	CL_Display::fill_triangle(x1+menu_left,y1+menu_top, x2+menu_left,y2+menu_top, x3+menu_left,y3+menu_top, gradient);
	CL_Display::fill_triangle(x1+menu_right,y1+menu_bottom, x2+menu_right,y2+menu_bottom, x3+menu_right,y3+menu_bottom, gradient);
	
	temp_angle = 360.0 - angle;
	x1=sin(temp_angle*DEG_TO_RAD)*triangle_radius;
	y1=cos(temp_angle*DEG_TO_RAD)*triangle_radius;
	x2=sin((temp_angle+120)*DEG_TO_RAD)*triangle_radius;
	y2=cos((temp_angle+120)*DEG_TO_RAD)*triangle_radius;
	x3=sin((temp_angle+240)*DEG_TO_RAD)*triangle_radius;
	y3=cos((temp_angle+240)*DEG_TO_RAD)*triangle_radius;
	
	CL_Display::fill_triangle(x1+menu_right,y1+menu_top, x2+menu_right,y2+menu_top, x3+menu_right,y3+menu_top, gradient);
	CL_Display::fill_triangle(x1+menu_left,y1+menu_bottom, x2+menu_left,y2+menu_bottom, x3+menu_left,y3+menu_bottom, gradient);
	
	
	// Menu rectangle
	CL_Display::fill_rect(menu_rect, gradient);
	CL_Display::draw_rect(menu_rect, CL_Color(0, 0, 0, 255*visibility));
	
	if (current_tab == 0 && current_subtab[0] == 3 && full_visibility) {
		// If current menu is General/Audio
		CL_Color boxcol;
		for (int i = 0; i < NUMBER_OF_BARS; i++) {
			if (i < sfx_bars_used)
				boxcol = CL_Color(0,0,0);
			else
				boxcol = CL_Color(128,128,128);
			CL_Display::fill_rect(sfx_boxes[i], boxcol);
		}
	}
	
	     // ´´´´´´´´´´´´´´´´´´´´´´´´´´´´ //
	    //    Actually _show_ the GUI   //
	if (full_visibility) gui->show();  //
	  //                              //
	 //////////////////////////////////
	
	
	// Calculate new color based on current color
	if (color_r == 255 && color_g < 255 && color_b == 0)
		color_g = color_g + COLOR_SPEED < 256 ? color_g + COLOR_SPEED : 255;
	else if (color_r > 0 && color_g == 255 && color_b == 0)
		color_r = color_r - COLOR_SPEED >= 0 ? color_r - COLOR_SPEED : 0;
	else if (color_r == 0 && color_g == 255 & color_b < 255)
		color_b = color_b + COLOR_SPEED < 256 ? color_b + COLOR_SPEED : 255;
	else if (color_r == 0 && color_g > 0 && color_b == 255)
		color_g = color_g - COLOR_SPEED >= 0 ? color_g - COLOR_SPEED : 0;
	else if (color_r < 255 && color_g == 0 && color_b == 255)
		color_r = color_r + COLOR_SPEED < 256 ? color_r + COLOR_SPEED : 255;
	else if (color_r == 255 && color_g == 0 && color_b > 0)
		color_b = color_b - COLOR_SPEED >= 0 ? color_b - COLOR_SPEED : 0;
	
	// Increase triANGLE
	angle += 4;
	
	// Check if leaving, arriving or staying
	if (increase_visibility) {
		if (!full_visibility) {
			visibility += .2;
			if (visibility > 1.0) {
				visibility = 1.0;
				full_visibility = true;
			}
		}
	}
	else {
		visibility -= .2;
		if (visibility < 0.0) {
			visibility = 0.0;
			active = false;
		}
	}
}
