/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         player.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Player object to control player variables,
 *                          components, etc.
 *
 */

#include "player.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Player::Player(float x, float y, float angle, World *world, int id)
: GameObject(x, y, angle, 0, 0, world), id(id) {
	init = true;
	alive = true;
	does_control = false;
	hspeed = 0;
	vspeed = 0;
	collision_timer = CL_System::get_time() / 1000.0f;
	collision = false;
	radius = 200;
	
	sprite_collision = new CL_Sprite("Poddie/Shield/Blue1", world->resources);
	
	for (int i = 0; i < 16; i++) slot[i] = false;
	/*
	components.push_back(new Component(50, "Rusty", "Weapon/Laser", "FrontRight1", this, world));
	components.push_back(new Component(50, "Rusty", "Weapon/Laser", "FrontLeft1", this, world));
	components.push_back(new Component(35, "Rusty", "Weapon/Laser", "BackCenter1", this, world));
	components.push_back(new Component(37, "Rusty", "Accessories", "Assistant1", this, world));
	*/
}

Player::Player() {
}

Player::~Player() {
	for (list<Component*>::iterator it = components.begin(); it != components.end(); it++) {
		delete *it;
	}
	components.clear();
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            LOOPING FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Player::update() {
	
	// Do the basic physics & movement calculations.
	x += hspeed;
	y += vspeed;
	apply_friction(1.005);
	
	speed = sqrt(pow(hspeed, 2)+pow(vspeed, 2));
	
	// Determine when we should stop drawing our shield.
	if (collision && CL_System::get_time() / 1000.0f > collision_timer) {
		collision = false;
	}
	
	// Update our components.
	list<Component*>::iterator it; 
	for (it = components.begin(); it != components.end(); it++) {
		Component *cur = *it;
		cur->update();
	}
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                              DRAW FUNCTION
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Player::draw() {
	float draw_x = get_x()*world->zoom_factor-world->view_x;
	float draw_y = get_y()*world->zoom_factor-world->view_y;
	
	// Draw the components.
	list<Component*>::iterator it; 
	for (it = components.begin(); it != components.end(); it++) {
		Component *cur = *it;
		cur->sprite->set_scale(world->zoom_factor/world->quality, world->zoom_factor/world->quality);
		cur->sprite->set_alignment(origin_center);
		cur->sprite->set_angle(angle-90);
		if (cur->active) {
			cur->activitySprite->set_scale(world->zoom_factor/world->quality, world->zoom_factor/world->quality);
			cur->activitySprite->set_alignment(origin_center);
			cur->activitySprite->set_angle(angle-90);
			cur->activitySprite->set_alpha(cur->activity);
			cur->activitySprite->draw(draw_x, draw_y, world->window->get_gc());
		}
		cur->sprite->draw(draw_x, draw_y, world->window->get_gc());
	}
	
	// Draw a shield upon collisions.
	if (collision) {
		sprite_collision->set_alignment(origin_center);
		sprite_collision->set_alpha(((collision_timer - CL_System::get_time() / 1000.0f)/0.6f)/1.5f);
		float scale = (world->zoom_factor/world->quality)*1.0f/1024.0f*radius*2;
		sprite_collision->set_scale(scale, scale);
		sprite_collision->draw(draw_x, draw_y, world->window->get_gc());
	}
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

////////////////////////////////////////////////////////////////////////
//                     MOVEMENT, POSITION & DIRECTION

void Player::set_angle(float new_angle) { angle = new_angle; }
void Player::set_hspeed(float new_hspeed) { hspeed = new_hspeed; }
void Player::set_vspeed(float new_vspeed) { vspeed = new_vspeed; }
void Player::set_x(float new_x) { x = new_x; }
void Player::set_y(float new_y) { y = new_y; }

float Player::get_speed() {
	return speed;
}

//                           SIMPLE PHYSICS

void Player::add_vector(float angle, float acceleration) {
	vspeed += sin(angle*DEG_TO_RAD)*acceleration;
	hspeed += cos(angle*DEG_TO_RAD)*acceleration;
	//cout << speed << endl;
}

void Player::apply_friction(float friction) {
	vspeed /= friction;
	hspeed /= friction;
}

////////////////////////////////////////////////////////////////////////
//                             COMPONENTS

//                                ADD
void Player::add_component(list<Component*>::iterator pos, Component *comp) {
	components.insert(pos, comp);
}

//                               REMOVE
void Player::remove_component(list<Component*>::iterator pos) {
	components.erase(pos);
}

////////////////////////////////////////////////////////////////////////
//                            TEXT STRINGS

string Player::get_speed_string() {
	int decimal_precision = 3;
	float display_speed = round((double)speed*pow((double)10, (double)decimal_precision))/pow((double)10, (double)decimal_precision)*10;
	
	ostringstream ss;
	string speed_str;
	ss << "Speed: " << display_speed;
	speed_str = ss.str();
	return speed_str;
}

string Player::get_position_string() {	
	ostringstream ss;
	string position_str;
	ss << "Position: " << (int)x << ", " << (int)y;
	position_str = ss.str();
	return position_str;
}

