/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poddie is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * A copy of the GNU General Public License can be found in the file
 * LICENSE. 
 */

/*
 *                         world.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Controls all major functions of the game,
 *                          and takes care of updating objects etc.
 *
 */

#include "world.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

// Constructor
World::World(CL_ResourceManager *resources, CL_DisplayWindow *window, ConfigFile config, bool debug) :
	resources(resources), window(window), config(config), debug(debug) {
	
	// Preload resources
	resources->load_section("Game");
	resources->load_section("Poddie");
	resources->load_section("Missile");
	
	// Define the quality scale value
	if (config.read<string>("graphics_quality") == "low") quality = 0.5; // Low
	else if (config.read<string>("graphics_quality") == "verylow") quality = 0.25; // Very Low
	else quality = 1; // High
	
	// FPS
	FPS = config.read<float>("FPS");
	FPS_count = CL_System::get_time()+1000/FPS;
	FPS_last = CL_System::get_time();
	frames = 0;
	FPS_cur = 0;
	FPS_string = "";

	// Sound configuration
	sfx_volume = config.read<float>("sfx_volume");
	
	// Network connections
	do_connect = false;
	can_connect = true;
	has_connected = false;
	
	// Player control
	has_control = false;
	i_control = -1;
	
	// Network delay
	last_call_delay = CL_System::get_time();
	request_next_delay = CL_System::get_time();
	
	// Set room values
	room = "GAME MENU";
	
	// Initialize camera view
	view_x = 0;
	view_y = 0;
	zoom_factor = (float)window->get_width()/((float)1280*4);
	
	// Surface
	surface_hrepeat = 15;
	surface_vrepeat = 15;
	surface_scale = 3/quality;
	
	// Minimap
	minimap_range = 200;
	update_minimap();
	
	// If true, the game will end.
	quit = false;
}

// Destructor
World::~World() {
		
	// Unload resources to prevent segfaults on exit.
	resources->unload_section("Game");
	resources->unload_section("Poddie");
	resources->unload_section("Missile");
	
	// Clear the object arrays to free memory.
	player.clear();
	missile.clear();
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                          THE ACTUAL GAME LOOP
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void World::run() {
	////////////////////////////////////////////////////////////////////
	//                       LOADING & SETUP
	
	// Read the server list file.
	cout << "Reading server list..." << endl << endl; 
	
	// Variables
	string server_line;
	string server_tmp_line;
	
	string server_ip_line;
	string server_port_line;
	string server_name_line;
	
	// Create an input stream.
	ifstream server_list_file("config/servers.list");
	
	if (server_list_file.is_open()) {
		while (!server_list_file.eof()) { // Repeat until we reach the end of the file (EOF).
			getline(server_list_file, server_line);
			
			// Find the server name
			size_t pos = server_line.find("\t", 0); // Search for the first occurence of a tab in the line and store its position.
			server_tmp_line = server_line.substr(0, pos); // Extract the IP using the tab position.
			server_name_line = server_line.substr(pos+1); // Extract the name using the tab position.
			server_name.push_back(server_name_line); // Store the rest of the line as a server name in our server_name vector.
			
			// Find the IP and port
			pos = server_tmp_line.find(":", 0); // Search for the first occurence of a colon in the IP line and store its position.
			server_ip_line = server_tmp_line.substr(0, pos); // Get the IP string by stripping out the colon.
			server_ip.push_back(server_ip_line); // Store the IP in our server_ip vector.
			server_port_line = server_tmp_line.substr(pos+1); // Extract the port using the colon position.
			server_port.push_back(server_port_line); // Store the IP in our server_ip vector.
			
			cout << "\t" << server_ip_line << ":" << server_port_line << " - " << server_name_line << endl;
		}
		server_list_file.close();
	}
	// Print an error if the server list couldn't be read and set the default server to localhost.
	else {
		cout << "Couldn't read server list." << endl;
		
		server_ip.push_back("localhost");
		server_port.push_back("7223");
		server_name.push_back("Localhost");
		
		cout << "Localhost is set as the only server." << endl;
	}
	
	cout << endl << "Starting netsession... \t";
	
	//Create the netsession.
	CL_NetSession netsession("Poddie");
	
	cout << " Success." << endl;
	cout << "Start listening for packets \t";
	
	//Listen for packets on the following channels:
	slots.connect(netsession.sig_netpacket_receive("object_status"), this, &World::network_object_status_receive);
	slots.connect(netsession.sig_netpacket_receive("missile_create"), this, &World::network_missile_create_receive);
	slots.connect(netsession.sig_netpacket_receive("collision"), this, &World::network_collision_receive);
	slots.connect(netsession.sig_netpacket_receive("component_add"), this, &World::network_component_add_receive);
	slots.connect(netsession.sig_netpacket_receive("component_remove"), this, &World::network_component_remove_receive);
	slots.connect(netsession.sig_netpacket_receive("component_activate"), this, &World::network_component_activate_receive);
	slots.connect(netsession.sig_netpacket_receive("component_activity"), this, &World::network_component_activity_receive);
	slots.connect(netsession.sig_netpacket_receive("delay"), this, &World::network_delay_receive);
	slots.connect(netsession.sig_netpacket_receive("alive"), this, &World::network_alive_receive);
	slots.connect(netsession.sig_netpacket_receive("dead"), this, &World::network_dead_receive);
	
	// Listen for keyboard input.
	slots.connect((window->get_ic()->get_keyboard()).sig_key_down(), this, &World::keyboard_pressed);
	slots.connect((window->get_ic()->get_keyboard()).sig_key_up(), this, &World::keyboard_released);
	
	cout << " Success." << endl;
	cout << "Setting up fonts... \t";
	
	// Setup the fonts.
	CL_Font monospace("Game/Font/Monospace", resources);
	monospace.set_color(CL_Color(0, 0, 0));
	
	cout << " Success." << endl;
	cout << "Creating surface... \t";
	
	// Create the surface.
	CL_Surface surface("Game/Surface/Moon1", resources);
	surface.set_alignment(origin_top_left);
	surface.set_scale(zoom_factor*surface_scale, zoom_factor*surface_scale);
	
	cout << " Success." << endl;
	
	cout << "Preparing components menu... \t";
	
	// Prepare the basic GUI system.
	CL_ResourceManager comp_res("resources/GUI/GUIStyleSilver/gui.xml");
	CL_StyleManager_Silver comp_style(&comp_res);
	CL_GUIManager comp_gui(&comp_style);
	
	ComponentsMenu comp_menu(this, &comp_gui);
	
	cout << " Success." << endl;
	cout << "Preparing game menu... \t";
	
	// Prepare the basic GUI system.
	CL_ResourceManager menu_res("resources/GUI/GUIStyleSilver/gui.xml");
	CL_StyleManager_Silver menu_style(&menu_res);
	CL_GUIManager menu_gui(&menu_style);
	
	GameMenu menu(this, &menu_gui, &comp_menu);
	
	cout << " Success." << endl;
	
	
	// Set the default server IP and port to the first server instance.
	string ip = server_ip[0];
	string port = server_port[0];
	
	////////////////////////////////////////////////////////////////////
	//                         GAME LOOP
	
	while (!quit) {
		
		//--------------------------------------------------------------
		//!                GLOBAL GAME EVENTS
		//--------------------------------------------------------------
		
		//Quit upon pressing escape
		if (keyboard.released(CL_KEY_ESCAPE)) quit = true;
		
		//--------------------------------------------------------------
		//!                SERVER CONNECTION
		//--------------------------------------------------------------
		
		if (do_connect) {
			cout << "Connecting to server at: " << ip << ":" << port << " ... \t";
		
			//Establish a connection to the server with the given IP and port.
			server = netsession.connect(CL_IPAddress(ip, port));
			
			cout << " Success." << endl;
			has_connected = true;
			do_connect = false;
			can_connect = false;
		}
		
		// If we are connected and controls a player.
		if (has_connected && i_control >= 0) {
			
			//Ping the server with packets to get an almost accurate delay
			if (request_next_delay <= CL_System::get_time()) {
				network_delay_return_send(i_control);
				request_next_delay = CL_System::get_time()+2000;
			}
		}
		
		//--------------------------------------------------------------
		//!                FPS CALCULATION
		//--------------------------------------------------------------
		
		if (CL_System::get_time() > FPS_last) {
			FPS_cur = frames;
			FPS_last = CL_System::get_time()+1000;
			frames = 0;
			ostringstream ss;
			ss << "FPS: " << FPS_cur;
			FPS_string = ss.str();
		}
		
		//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
		//                     ROOMS
		//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
		
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		
		/// ////////////////////////////////////////////////////////////
		///               ROOM: GAME MENU                             //
		///                                                           //
		
		if (room == "GAME MENU") {
			
			// Clear the screen with a white background.
			CL_Display::clear(CL_Color(255,255,255));
			
			monospace.set_alignment(origin_top_center);
			
			for (int i = 0; i < server_ip.size(); i++) {
				// Calculate if the mouse is hovering the button.
				if (CL_Mouse::get_y() > 200+100*i && CL_Mouse::get_y() < 200+100*(i+1)-76) {
					
					// Set the colour to light grey.
					monospace.set_color(CL_Color(128, 128, 128));
					
					// Join the hovered server if left mouse button is clicked.
					if (CL_Mouse::get_keycode(CL_MOUSE_LEFT) && can_connect) {
						do_connect = true;
						ip = server_ip[i];
						port = server_port[i];
						room = "TEST GAME";
					}
				}
				// Else set the colour to black.
				else monospace.set_color(CL_Color(0, 0, 0));
				
				// Draw the server name onto the screen.
				monospace.draw(window->get_width()/2, 200+100*i, server_name[i]);
			}
		
			/* Flip the display, showing on the screen what we have drawed
			 * since last call to flip(). */
			CL_Display::flip();
		}
		
		///                                                           //
		/// ////////////////////////////////////////////////////////////
		
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		
		/// ////////////////////////////////////////////////////////////
		///               ROOM: TEST GAME                             //
		///                                                           //
		
		if (room == "TEST GAME") {
			
			if (i_control >= 0) { // Requires control over a player.
				
				/// //////////////////////////////////////////////// ///
				///          TEMPORARY CODE                          ///
				/// //////////////////////////////////////////////// ///
				
				//! TODO: Rewrite the way components are being added.
				if (keyboard.pressed(CL_KEY_Z)) network_component_add_send(player.at(i_control)->id, "Rusty", "Base", "Main1", 0);
				if (keyboard.pressed(CL_KEY_C)) {
					if (!player.at(i_control)->slot[1]) network_component_add_send(player.at(i_control)->id, "Rusty", "Armour", "Main1", 1);
					if (!player.at(i_control)->slot[2]) network_component_add_send(player.at(i_control)->id, "Rusty", "Armour", "Front1", 2);
					if (!player.at(i_control)->slot[6]) network_component_add_send(player.at(i_control)->id, "Rusty", "Armour", "Wings1", 6);
					if (!player.at(i_control)->slot[7]) network_component_add_send(player.at(i_control)->id, "Rusty", "Armour", "WingsFront1", 7);
				}
				if (keyboard.pressed(CL_KEY_X)) {
					if (!player.at(i_control)->slot[3]) network_component_add_send(player.at(i_control)->id, "Rusty", "Engine", "BackCenter1", 3);
					if (!player.at(i_control)->slot[4]) network_component_add_send(player.at(i_control)->id, "Rusty", "Engine", "BackRight1", 4);
					if (!player.at(i_control)->slot[5]) network_component_add_send(player.at(i_control)->id, "Rusty", "Engine", "BackLeft1", 5);
					if (!player.at(i_control)->slot[8]) network_component_add_send(player.at(i_control)->id, "Rusty", "Engine", "MiddleRight1", 8);
					if (!player.at(i_control)->slot[9]) network_component_add_send(player.at(i_control)->id, "Rusty", "Engine", "MiddleLeft1", 9);
				}
				if (keyboard.pressed(CL_KEY_V)) {
					if (!player.at(i_control)->slot[10]) network_component_add_send(player.at(i_control)->id, "Rusty", "Weapon/Laser", "MiddleRight1", 10);
					if (!player.at(i_control)->slot[11]) network_component_add_send(player.at(i_control)->id, "Rusty", "Weapon/Laser", "MiddleLeft1", 11);
				}
				if (keyboard.pressed(CL_KEY_M) && !comp_menu.active)
					menu.change_state();
				if (keyboard.pressed(CL_KEY_N) && !menu.active)
					comp_menu.change_state();
				
				/// //////////////////////////////////////////////// ///
				///        END: TEMPORARY CODE                       ///
				/// //////////////////////////////////////////////// ///
				
				//------------------------------------------------------
				//!        VIEWPORT CALCULATIONS
				//------------------------------------------------------
				
				//!              ZOOMING
				
				// Zoom in...
				if (keyboard.is_pressed(CL_KEY_NUMPAD_PLUS)) {
					zoom_factor *= 1.005;
					// Scale the surface, since this is not otherwise being done.
					surface.set_scale(zoom_factor*surface_scale, zoom_factor*surface_scale);
				}
				
				// Zoom out...
				if (keyboard.is_pressed(CL_KEY_NUMPAD_MINUS)) {
					zoom_factor /= 1.005;
					// Scale the surface, since this is not otherwise being done.
					surface.set_scale(zoom_factor*surface_scale, zoom_factor*surface_scale);
				}
				
				//!             VIEWPORT
				
				// Center the view at our poddie.
				//! TODO: View should be controlled by the player's probe, not his poddie.
				view_x = window->get_width()/2-window->get_width()+player.at(i_control)->get_x()*zoom_factor;
				view_y = window->get_height()/2-window->get_height()+player.at(i_control)->get_y()*zoom_factor;
				
				
		
				//------------------------------------------------------
				//!        UPDATE GAME OBJECTS
				//------------------------------------------------------
				
				//!             MISSILES
				for (int i = 0; i < missile.size(); i++)
					if (missile.at(i)->alive || missile.at(i)->collision)
						missile.at(i)->update();
				
				//!              PLAYERS
				for (int i = 0; i < player.size(); i++)
					if (player.at(i)->alive)
						player.at(i)->update();
				
				//------------------------------------------------------
				//!       DRAW ONTO THE SCREEN
				//------------------------------------------------------
				
				// Only draw if we do not pass the FPS limit.
				if (FPS_count < CL_System::get_time()) {
					
					// Set the counter for the next draw action.
					FPS_count = CL_System::get_time()+1000/FPS;
					
					// Clear the screen with a black background.
					CL_Display::clear(CL_Color(0, 0, 0));
					
					//!         SURFACE
					
					// Repeat the surface
					for (int h = 0; h < surface_hrepeat; h++) {
						for (int v = 0; v < surface_vrepeat; v++) {
							
							float x = h*surface.get_width()*zoom_factor*surface_scale-view_x;
							float y = v*surface.get_height()*zoom_factor*surface_scale-view_y;
							
							surface.draw(x, y, window->get_gc());
						}
					}
					
					//!        MISSILES
					for (int i = 0; i < missile.size(); i++)
						if (missile.at(i)->alive || missile.at(i)->collision)
							missile.at(i)->draw();
							
					//!         PLAYERS
					for (int i = 0; i < player.size(); i++)
						if (player.at(i)->alive)
							player.at(i)->draw();
					
					//--------------------------------------------------
					//!           GUI
					//--------------------------------------------------
					
					//! TODO: Rewrite the entire GUI into a new class.
					
					//!         MINIMAP
					draw_minimap(surface);
					
					//!     ACTIVITY STATES
					draw_activity();
					
					//!     ... INFORMATION
					if (player.size() >= i_control) {
						monospace.set_color(CL_Color(0, 0, 0));
						monospace.set_alignment(origin_top_left);
						monospace.draw(20, 20, player.at(i_control)->get_speed_string(), window->get_gc());
						monospace.draw(20, 50, player.at(i_control)->get_position_string(), window->get_gc());
						monospace.set_alignment(origin_top_right);
						monospace.draw(window->get_width()-20, 20, server_delay_string, window->get_gc());
						monospace.draw(window->get_width()-20, 50, FPS_string, window->get_gc());
					}
					
					//!           MENU
					if (menu.active)
						menu.draw();
					if (comp_menu.active)
						comp_menu.draw();
					
					// Increment the frames drawed.
					frames++;
		
					/* Flip the display, showing on the screen what we have drawed
					 * since last call to flip(). */
					CL_Display::flip();
				}
			}
		}
		
		///                                                           //
		/// ////////////////////////////////////////////////////////////
		
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////
		
		/* Update the keyboard - this function resets key-actions, such
		 * as presses and releases, so they don't occur twice. */
		keyboard.update();
		
		// Keep the game alive and wait 10 ms.
		CL_System::keep_alive(10);
	}
	
	////////////////////////////////////////////////////////////////////
	//                     QUITTING THE GAME
	
	if (has_connected) {
		cout << "Disconnecting from server... \t";
		
		CL_NetPacket *msg;
		msg = new CL_NetPacket();
		msg->output.write_int32(i_control);
		server.send("disconnect", *msg);
		delete msg;	// discard sent message
		
		// Wait a second before closing.
		CL_System::sleep(1000);
		
		server.disconnect("Client closed");
		
		cout << " Success." << endl;
	}
}


/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               NETWORKING
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                          IN-GAME RELATED
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

////////////////////////////////////////////////////////////////////////
//                              SENDING

//----------------------------------------------------------------------
//!                            COMPONENT
//----------------------------------------------------------------------

//                             ACTIVATE
void World::network_component_activate_send(int controller, string func, bool active, int slot) {
	//cout << "Sending request to change component's active state." << endl;
	
	packet_component_activate = new CL_NetPacket();
	packet_component_activate->output.write_int32(controller); //Tell the server which player we control.
	packet_component_activate->output.write_string(func);
	packet_component_activate->output.write_bool8(active);
	packet_component_activate->output.write_int32(slot);
	server.send("component_activate", *packet_component_activate);
	delete packet_component_activate;
}

//                                ADD
void World::network_component_add_send(int controller, string type, string part, string number, int slot) {
	//cout << "Sending request to add component." << endl;
	
	packet_component_add = new CL_NetPacket();
	packet_component_add->output.write_int32(controller); //Tell the server which player we control.
	packet_component_add->output.write_string(type);
	packet_component_add->output.write_string(part);
	packet_component_add->output.write_string(number);
	packet_component_add->output.write_int32(slot);
	server.send("component_add", *packet_component_add);
	delete packet_component_add;
}

//                               REMOVE
void World::network_component_remove_send(int controller, int slot) {
	cout << "Sending request to remove component." << endl;
	
	packet_component_remove = new CL_NetPacket();
	packet_component_remove->output.write_int32(controller); //Tell the server which player we control.
	packet_component_remove->output.write_int32(slot);
	server.send("component_remove", *packet_component_remove);
	delete packet_component_remove;
}

//----------------------------------------------------------------------
//!                              DELAY
//----------------------------------------------------------------------

void World::network_delay_return_send(int controller) {
	//cout << "Sending delay-packet." << endl;
	
	packet_delay_return = new CL_NetPacket();
	packet_delay_return->output.write_int32(controller); //Tell the server which player we control.
	server.send("delay_return", *packet_delay_return);
	last_call_delay = CL_System::get_time();
	delete packet_delay_return;
}

////////////////////////////////////////////////////////////////////////
//                             RECEIVING

//----------------------------------------------------------------------
//!                  MOVEMENT, POSITION & DIRECTION
//----------------------------------------------------------------------

//                           OBJECT STATUS
void World::network_object_status_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int alive = packet.input.read_bool8();
		int id = packet.input.read_int32();
		int object = packet.input.read_int32();
		float hspeed = packet.input.read_float32();
		float vspeed = packet.input.read_float32();
		float angle = packet.input.read_float32();
		float x = packet.input.read_float32();
		float y = packet.input.read_float32();
		
		//!                     MISSILE
		if (object == MISSILE) {
			missile.at(id)->alive = alive;
			missile.at(id)->set_hspeed(hspeed);
			missile.at(id)->set_vspeed(vspeed);
			missile.at(id)->set_x(x);
			missile.at(id)->set_y(y);
			missile.at(id)->set_angle(angle);
		}
		
		//!                      PLAYER
		if (object == PLAYER) {
			player.at(id)->set_hspeed(hspeed);
			player.at(id)->set_vspeed(vspeed);
			player.at(id)->set_x(x);
			player.at(id)->set_y(y);
			player.at(id)->set_angle(angle);
	
		}
	}	
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//----------------------------------------------------------------------
//!                             MISSILE
//----------------------------------------------------------------------

//                              CREATE
void World::network_missile_create_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		bool alive = packet.input.read_bool8();
		int id = packet.input.read_int32();
		float x = packet.input.read_float32();
		float y = packet.input.read_float32();
		float angle = packet.input.read_float32();
		float hspeed = packet.input.read_float32();
		float vspeed = packet.input.read_float32();
		string str_sprite = packet.input.read_string();
		
		CL_Sprite sprite(str_sprite, resources);
		missile.push_back(new Missile(alive, x, y, angle, hspeed, vspeed, player.at(id), this, sprite));
	}	
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//----------------------------------------------------------------------
//!                            COLLISION
//----------------------------------------------------------------------

void World::network_collision_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int id = packet.input.read_int32();
		int object = packet.input.read_int32();
		int radius = packet.input.read_float32();
		
		if (object == MISSILE) {
			missile.at(id)->collision = true;
			missile.at(id)->radius = radius;
			missile.at(id)->collision_timer = 0.3f + CL_System::get_time() / 1000.0f;
		}
		
		if (object == PLAYER) {
			player.at(id)->collision = true;
			player.at(id)->radius = radius;
			player.at(id)->collision_timer = 0.6f + CL_System::get_time() / 1000.0f;
		}
	}	
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//----------------------------------------------------------------------
//!                            COMPONENT
//----------------------------------------------------------------------

//                                ADD
void World::network_component_add_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int id = packet.input.read_int32();
		string type = packet.input.read_string();
		string part = packet.input.read_string();
		string number = packet.input.read_string();
		int slot = packet.input.read_int32();
		
		if (!player.at(id)->slot[slot]) { // Only add component, if slot is not already filled.
			player.at(id)->slot[slot] = true;
			cout << "Added component: \"" << type << ", " << part << ", " << number << "\" to Player " << id << endl;
			
			// Read the position at which we should insert the new component, based on its z-value.
			Component temp_comp(type, part, number, slot, *player.at(id)); //Create a temporary component...
			float z = temp_comp.z; // And read the z-value it would generate.
			list<Component*>::iterator it; // Use this to read through the components.
			list<Component*>::iterator pos = player.at(id)->components.begin(); // The position at which we will insert the component.
			for (it = player.at(id)->components.begin(); it != player.at(id)->components.end(); it++) {
				Component *cur = *it;
				if (z >= cur->z) { // We position values with a higher z, at the end of the list.
					pos++;
				}
			}
			
			Component *comp;
			comp = new Component(type, part, number, slot, *player.at(id)); // Create a new component...
			player.at(id)->add_component(pos, comp); // Add the component.
			
			player.at(id)->component[slot] = --pos; // Assign the iterator for the slot to the component we've just added.
		}
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//                               REMOVE
void World::network_component_remove_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int id = packet.input.read_int32();
		int slot = packet.input.read_int32();
		
		if (player.at(id)->slot[slot]) {
			player.at(id)->slot[slot] = false;
			cout << "Removed component at slot: \"" << slot << "\" from Player " << id << endl;
			player.at(id)->remove_component(player.at(id)->component[slot]);
		}
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//                              ACTIVATE
void World::network_component_activate_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int id = packet.input.read_int32();
		string func = packet.input.read_string();
		bool state = packet.input.read_bool8();
		int slot = packet.input.read_int32();
		
		list<Component*>::iterator it; 
		for (it = player.at(id)->components.begin(); it != player.at(id)->components.end(); it++) {
			Component *cur = *it;
			if (cur->slot == slot) {
				if (func == "active") cur->active = state;
				if (func == "increase") cur->increase = state;
				if (func == "decrease") cur->decrease = state;
			}
		}
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

//                              ACTIVITY
void World::network_component_activity_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	try {
		int id = packet.input.read_int32();
		float component_activity = packet.input.read_float32();
		int slot = packet.input.read_int32();
		
		list<Component*>::iterator it; 
		for (it = player.at(id)->components.begin(); it != player.at(id)->components.end(); it++) {
			Component *cur = *it;
			if (cur->slot == slot) {
				cur->activity = component_activity;
			}
		}
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}

}

//----------------------------------------------------------------------
//!                              DELAY
//----------------------------------------------------------------------

void World::network_delay_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	int id = packet.input.read_int32();
	float cur_time = CL_System::get_time();
	if (i_control == id) server_delay = cur_time - last_call_delay;
	
	//Convert the delay to a displayable string.
	ostringstream ss;
	ss << "Delay: " << server_delay << "ms";
	server_delay_string = ss.str();
}

//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//                         CONNECTION RELATED
//!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

////////////////////////////////////////////////////////////////////////
//                            CONNECTIONS

//----------------------------------------------------------------------
//!                             CONNECT
//----------------------------------------------------------------------

void World::network_alive_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	int id = packet.input.read_int32();
	bool does_control = packet.input.read_bool8();
	bool is_alive = packet.input.read_bool8();
	
	try {
		Player *new_player;
		new_player = new Player(0, 0, 0, this, id);
		player.push_back(new_player);
		player.at(id)->alive = is_alive;
		
		if (does_control && !has_control) {
			player.at(id)->does_control = true;
			i_control = id;
			has_control = true;
			cout << "You control: " << id << endl;
		}
		if (player.at(id)->alive) cout << "Player " << id << " is alive." << endl;
		
		//Request components for the player. This is to prevent packets about player components to arrive before the players are created.
		if (id == i_control) {
			CL_NetPacket *msg;
			msg = new CL_NetPacket();
			msg->output.write_int32(id);
			computer.send("requestComponents", *msg);
			delete msg;	// discard sent message
			cout << "Sent request for components." << endl;
		}
		
		if (player.at(id)->alive) cout << "Player " << id << " is alive." << endl;
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}

}

//----------------------------------------------------------------------
//!                            DISCONNECT
//----------------------------------------------------------------------

void World::network_dead_receive(CL_NetPacket &packet, CL_NetComputer &computer) {
	int id = packet.input.read_int32();
	try {
		player.at(id)->alive = false;
		player.at(id)->update(); //Update to allow the player and his components to die nicely.
		cout << "Player " << id << " is dead." << endl;
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                           UPDATE FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void World::update_minimap() {
	minimap_width = window->get_width()/4; //The minimap is 1/4th of the width.
	minimap_height = window->get_width()/4; //The minimap is 1/4th of the width.
	minimap_top = window->get_height()-minimap_height;
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                             DRAW FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

//                              MINIMAP
void World::draw_minimap(CL_Surface &surface) {
	// Draw the background for the minimap.
	CL_Display::fill_rect(CL_Rect(0, window->get_height()-minimap_height, minimap_width, window->get_height()), CL_Color(128, 128, 128, 128));
	
	// Draw map borders on the minimap.
	int border_left = (0-player.at(i_control)->get_x())/minimap_range+minimap_width/2;
	int border_top = (0-player.at(i_control)->get_y())/minimap_range+minimap_height/2;
	int border_right = (surface_hrepeat*surface.get_width()*surface_scale-player.at(i_control)->get_x())/minimap_range+minimap_width/2;
	int border_bottom = (surface_vrepeat*surface.get_height()*surface_scale-player.at(i_control)->get_y())/minimap_range+minimap_height/2;
	
	// Make sure the border corners do not exceed the minimap.
	if (border_right > minimap_width) border_right = minimap_width;
	if (border_left > minimap_width) border_left = minimap_width;
	if (border_bottom < 0) border_bottom = 0;
	if (border_top < 0) border_top = 0;
	
	// Do not draw borders if they are outside the minimap.
	if (border_left < minimap_width && border_left > 0)
		CL_Display::draw_line(border_left, minimap_top+border_top, border_left, minimap_top+border_bottom, CL_Color(192, 192, 255, 255));
	if (border_top < minimap_height && border_top > 0)
		CL_Display::draw_line(border_left, minimap_top+border_top, border_right, minimap_top+border_top, CL_Color(192, 192, 255, 255));
	if (border_right < minimap_width && border_right > 0)
		CL_Display::draw_line(border_right, minimap_top+border_top, border_right, minimap_top+border_bottom, CL_Color(192, 192, 255, 255));
	if (border_bottom < minimap_height && border_bottom > 0)
		CL_Display::draw_line(border_left, minimap_top+border_bottom, border_right, minimap_top+border_bottom, CL_Color(192, 192, 255, 255));
	
	//Draw the other players:
	for (int i = 0; i < player.size(); i++) if (player.at(i)->alive && i != i_control) {
		int drawX = (player.at(i)->get_x()-player.at(i_control)->get_x())/minimap_range+minimap_width/2;
		int drawY = (player.at(i)->get_y()-player.at(i_control)->get_y())/minimap_range+minimap_height/2;
		
		//The actual drawing:
		if (drawX < minimap_width-2 && drawY > 2) {
			CL_Display::fill_rect(CL_Rect(drawX-minimap_width/150, minimap_top+drawY-minimap_height/150, drawX+minimap_width/150, minimap_top+drawY+minimap_height/150), CL_Color(255, 0, 0, 255));
		}
	}
	
	//Draw our player object:
	CL_Display::fill_rect(CL_Rect(minimap_width/2-minimap_width/150, window->get_height()-minimap_height/2-minimap_height/150, minimap_width/2+minimap_width/150, window->get_height()-minimap_height/2+minimap_height/150), CL_Color(0, 255, 0, 255));
	CL_Display::draw_rect(CL_Rect(0, window->get_height()-minimap_height, minimap_width, window->get_height()), CL_Color(255, 255, 255, 255));
	
}

//                             ACTIVITY
void World::draw_activity() {
	try {
		//Draw the component_activity states of the controlled player
		int i = 0;
		list<Component*>::iterator comp;
		
		for (comp = player.at(i_control)->components.begin(); comp != player.at(i_control)->components.end(); comp++) {
			Component *cur = *comp;
			if (cur->part == "Engine") {
				CL_Display::fill_rect(CL_Rect(
								minimap_width+50+50*i,
								(int)(window->get_height()-100*cur->activity),
								minimap_width+70+50*i,
								window->get_height()),
								CL_Color(0, 0, 0, 255));
		
				CL_Display::draw_rect(CL_Rect(
								minimap_width+50+50*i,
								window->get_height()-100,
								minimap_width+70+50*i,
								window->get_height()), CL_Color(0, 0, 0, 255));
				i++;
			}
		}
	}
	catch (Exception &e) {
		cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
	}
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                           KEYBOARD FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

//                              PRESSED
void World::keyboard_pressed(const CL_InputEvent &key) {
	keyboard.set_pressed(key.id);
	//cout << "Keycode: " << key.id << endl; //Uncomment to retrieve keycode for "rare" keys.
}

//                              RELEASED
void World::keyboard_released(const CL_InputEvent &key) {
	keyboard.set_released(key.id);
}
